@@ -25,6 +25,7 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include "contrib/string.h"
 #include "libknot/errcode.h"
 
 static bool special_name(const char *name)
@@ -140,3 +141,44 @@ int make_path(const char *path, mode_t mode)
 
 	return KNOT_EOK;
 }
+
+int open_tmp_file(const char *path, char **tmp_name, FILE **file, mode_t mode)
+{
+	int ret;
+
+	*tmp_name = sprintf_alloc("%s.XXXXXX", path);
+	if (*tmp_name == NULL) {
+		ret = KNOT_ENOMEM;
+		goto open_tmp_failed;
+	}
+
+	int fd = mkstemp(*tmp_name);
+	if (fd < 0) {
+		ret = knot_map_errno();
+		goto open_tmp_failed;
+	}
+
+	if (fchmod(fd, mode) != 0) {
+		ret = knot_map_errno();
+		close(fd);
+		unlink(*tmp_name);
+		goto open_tmp_failed;
+	}
+
+	*file = fdopen(fd, "w");
+	if (*file == NULL) {
+		ret = knot_map_errno();
+		close(fd);
+		unlink(*tmp_name);
+		goto open_tmp_failed;
+	}
+
+	return KNOT_EOK;
+open_tmp_failed:
+	free(*tmp_name);
+	*tmp_name = NULL;
+	*file = NULL;
+
+	assert(ret != KNOT_EOK);
+	return ret;
+}
