@@ -91,7 +91,10 @@ enum dnssec_error {
 
 	DNSSEC_KEYSTORE_INVALID_BACKEND,
 	DNSSEC_KEYSTORE_INVALID_CONFIG,
-	DNSSEC_KEYSTORE_FAILED_TO_LOAD_P11_MODULE,
+
+	DNSSEC_P11_FAILED_TO_LOAD_MODULE,
+	DNSSEC_P11_TOO_MANY_MODULES,
+	DNSSEC_P11_TOKEN_NOT_AVAILABLE,
 
 	DNSSEC_ERROR_MAX = -1001
 };
