@@ -34,6 +34,9 @@ static const encode_attr_t POLICY_ATTRS[] = {
 	{ attr(rrsig_lifetime),       encode_uint32, decode_uint32 },
 	{ attr(rrsig_refresh_before), encode_uint32, decode_uint32 },
 	{ attr(nsec3_enabled),        encode_bool,   decode_bool   },
+	{ attr(nsec3_iterations),     encode_uint16, decode_uint16 },
+	{ attr(nsec3_salt_length),    encode_uint8,  decode_uint8  },
+	{ attr(nsec3_salt_lifetime),  encode_uint32, decode_uint32 },
 	{ attr(soa_minimal_ttl),      encode_uint32, decode_uint32 },
 	{ attr(zone_maximal_ttl),     encode_uint32, decode_uint32 },
 	{ attr(propagation_delay),    encode_uint32, decode_uint32 },
