@@ -20,7 +20,6 @@
 #include <stdint.h>
 
 #include "key.h"
-#include "keyid.h"
 #include "dname.h"
 
 /*!
@@ -30,16 +29,11 @@ struct dnssec_key {
 	uint8_t *dname;
 	dnssec_binary_t rdata;
 
-	char id[DNSSEC_KEYID_SIZE + 1];
-	uint16_t keytag;
-
 	gnutls_pubkey_t public_key;
 	gnutls_privkey_t private_key;
 	unsigned bits;
 };
 
-void key_update_identifiers(dnssec_key_t *key);
-
 static const uint16_t DNSKEY_FLAGS_KSK = 257;
 static const uint16_t DNSKEY_FLAGS_ZSK = 256;
 
