@@ -33,6 +33,7 @@
 #define C_ADDR			"\x07""address"
 #define C_ALG			"\x09""algorithm"
 #define C_ANY			"\x03""any"
+#define C_APPEND		"\x06""append"
 #define C_ASYNC_START		"\x0B""async-start"
 #define C_BACKEND		"\x07""backend"
 #define C_BG_WORKERS		"\x12""background-workers"
@@ -52,7 +53,8 @@
 #define C_IDENT			"\x08""identity"
 #define C_INCL			"\x07""include"
 #define C_IXFR_DIFF		"\x15""ixfr-from-differences"
-#define C_JOURNAL		"\x07""journal"
+#define C_JOURNAL		"\x07""journal" /* obsolete, old journal compat */
+#define C_JOURNAL_DB		"\x0A""journal-db"
 #define C_KASP_DB		"\x07""kasp-db"
 #define C_KEY			"\x03""key"
 #define C_KEYSTORE		"\x08""keystore"
@@ -61,7 +63,10 @@
 #define C_LOG			"\x03""log"
 #define C_MANUAL		"\x06""manual"
 #define C_MASTER		"\x06""master"
-#define C_MAX_JOURNAL_SIZE	"\x10""max-journal-size"
+#define C_MAX_JOURNAL_SIZE	"\x10""max-journal-size" /* obsolete, old journal compat */
+#define C_MAX_JOURNAL_DB_SIZE	"\x13""max-journal-db-size"
+#define C_MAX_JOURNAL_USAGE	"\x11""max-journal-usage"
+#define C_MAX_JOURNAL_DEPTH	"\x11""max-journal-depth"
 #define C_MAX_TCP_CLIENTS	"\x0F""max-tcp-clients"
 #define C_MAX_UDP_PAYLOAD	"\x0F""max-udp-payload"
 #define C_MAX_ZONE_SIZE		"\x0D""max-zone-size"
@@ -90,7 +95,10 @@
 #define C_SEM_CHECKS		"\x0F""semantic-checks"
 #define C_SERIAL_POLICY		"\x0D""serial-policy"
 #define C_SERVER		"\x06""server"
+#define C_SINGLE_TYPE_SIGNING	"\x13""single-type-signing"
 #define C_SRV			"\x06""server"
+#define C_STATS			"\x0A""statistics"
+#define C_TIMER			"\x05""timer"
 #define C_STORAGE		"\x07""storage"
 #define C_TARGET		"\x06""target"
 #define C_TCP_HSHAKE_TIMEOUT	"\x15""tcp-handshake-timeout"
