@@ -1,4 +1,4 @@
-/*  Copyright (C) 2014 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -162,27 +162,25 @@ int knot_nsec_chain_iterate_create(zone_tree_t *nodes,
 	assert(nodes);
 	assert(callback);
 
-	bool sorted = true;
-	hattrie_iter_t *it = hattrie_iter_begin(nodes, sorted);
-
+	trie_it_t *it = trie_it_begin(nodes);
 	if (!it) {
 		return KNOT_ENOMEM;
 	}
 
-	if (hattrie_iter_finished(it)) {
-		hattrie_iter_free(it);
+	if (trie_it_finished(it)) {
+		trie_it_free(it);
 		return KNOT_EINVAL;
 	}
 
-	zone_node_t *first = (zone_node_t *)*hattrie_iter_val(it);
+	zone_node_t *first = (zone_node_t *)*trie_it_val(it);
 	zone_node_t *previous = first;
 	zone_node_t *current = first;
 
-	hattrie_iter_next(it);
+	trie_it_next(it);
 
 	int result = KNOT_EOK;
-	while (!hattrie_iter_finished(it)) {
-		current = (zone_node_t *)*hattrie_iter_val(it);
+	while (!trie_it_finished(it)) {
+		current = (zone_node_t *)*trie_it_val(it);
 
 		result = callback(previous, current, data);
 		if (result == NSEC_NODE_SKIP) {
@@ -191,13 +189,13 @@ int knot_nsec_chain_iterate_create(zone_tree_t *nodes,
 		} else if (result == KNOT_EOK) {
 			previous = current;
 		} else {
-			hattrie_iter_free(it);
+			trie_it_free(it);
 			return result;
 		}
-		hattrie_iter_next(it);
+		trie_it_next(it);
 	}
 
-	hattrie_iter_free(it);
+	trie_it_free(it);
 
 	return result == NSEC_NODE_SKIP ? callback(previous, first, data) :
 	                 callback(current, first, data);
