@@ -1,4 +1,4 @@
-/*  Copyright (C) 2014 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,32 +13,25 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-/*!
- * \file nsec3-chain.h
- *
- * \author Jan Vcelak <jan.vcelak@nic.cz> (chain creation)
- *
- * \brief NSEC3 chain creation.
- *
- * \addtogroup dnssec
- * @{
- */
 
 #pragma once
 
-#include "knot/dnssec/zone-nsec.h"
-#include "knot/dnssec/nsec-chain.h"
+#include <stdint.h>
+#include "dnssec/nsec.h"
+#include "knot/updates/changesets.h"
+#include "knot/zone/contents.h"
 
 /*!
  * \brief Creates new NSEC3 chain, add differences from current into a changeset.
  *
  * \param zone       Zone to be checked.
+ * \param params     NSEC3 parameters.
  * \param ttl        TTL for new records.
  * \param changeset  Changeset to store changes into.
  *
  * \return KNOT_E*
  */
-int knot_nsec3_create_chain(const zone_contents_t *zone, uint32_t ttl,
+int knot_nsec3_create_chain(const zone_contents_t *zone,
+                            const dnssec_nsec3_params_t *params,
+                            uint32_t ttl,
                             changeset_t *changeset);
-
-/*! @} */
