@@ -15,6 +15,7 @@
 */
 
 #include <assert.h>
+#include <stdarg.h>
 #include <time.h>
 #include <urcu.h>
 
@@ -38,7 +39,6 @@ typedef struct event_info {
 static const event_info_t EVENT_INFO[] = {
 	{ ZONE_EVENT_LOAD,    event_load,    "load" },
 	{ ZONE_EVENT_REFRESH, event_refresh, "refresh" },
-	{ ZONE_EVENT_XFER,    event_xfer,    "transfer" },
 	{ ZONE_EVENT_UPDATE,  event_update,  "update" },
 	{ ZONE_EVENT_EXPIRE,  event_expire,  "expiration" },
 	{ ZONE_EVENT_FLUSH,   event_flush,   "journal flush" },
@@ -72,6 +72,28 @@ static time_t time_until(time_t planned)
 	return now < planned ? (planned - now) : 0;
 }
 
+/*!
+ * \brief Set time of a given event type.
+ */
+static void event_set_time(zone_events_t *events, zone_event_type_t type, time_t time)
+{
+	assert(events);
+	assert(valid_event(type));
+
+	events->time[type] = time;
+}
+
+/*!
+ * \brief Get time of a given event type.
+ */
+static time_t event_get_time(zone_events_t *events, zone_event_type_t type)
+{
+	assert(events);
+	assert(valid_event(type));
+
+	return events->time[type];
+}
+
 /*!
  * \brief Find next scheduled zone event.
  *
@@ -104,25 +126,12 @@ static zone_event_type_t get_next_event(zone_events_t *events)
 }
 
 /*!
- * \brief Set time of a given event type.
+ * \brief Fined time of next scheduled event.
  */
-static void event_set_time(zone_events_t *events, zone_event_type_t type, time_t time)
+static time_t get_next_time(zone_events_t *events)
 {
-	assert(events);
-	assert(valid_event(type));
-
-	events->time[type] = time;
-}
-
-/*!
- * \brief Get time of a given event type.
- */
-static time_t event_get_time(zone_events_t *events, zone_event_type_t type)
-{
-	assert(events);
-	assert(valid_event(type));
-
-	return events->time[type];
+	zone_event_type_t type = get_next_event(events);
+	return valid_event(type) ? event_get_time(events, type) : 0;
 }
 
 /*!
@@ -267,26 +276,38 @@ void zone_events_deinit(zone_t *zone)
 	memset(&zone->events, 0, sizeof(zone->events));
 }
 
-void zone_events_schedule_at(zone_t *zone, zone_event_type_t type, time_t time)
+void _zone_events_schedule_at(zone_t *zone, ...)
 {
-	if (!zone || !valid_event(type)) {
-		return;
-	}
-
 	zone_events_t *events = &zone->events;
+	va_list args;
+	va_start(args, zone);
 
 	pthread_mutex_lock(&events->mx);
-	time_t current = event_get_time(events, type);
-	if (time == 0 || current == 0 || time < current) {
-		event_set_time(events, type, time);
+
+	time_t old_next = get_next_time(events);
+
+	// update timers
+	for (int type = va_arg(args, int); valid_event(type); type = va_arg(args, int)) {
+		time_t planned = va_arg(args, time_t);
+		if (planned < 0) {
+			continue;
+		}
+
+		time_t current = event_get_time(events, type);
+		if (planned == 0 || current == 0 || planned < current) {
+			event_set_time(events, type, planned);
+		}
+	}
+
+	// reschedule if changed
+	time_t next = get_next_time(events);
+	if (old_next != next) {
 		reschedule(events);
 	}
-	pthread_mutex_unlock(&events->mx);
-}
 
-bool zone_events_is_scheduled(zone_t *zone, zone_event_type_t type)
-{
-	return zone_events_get_time(zone, type) > 0;
+	pthread_mutex_unlock(&events->mx);
+
+	va_end(args);
 }
 
 void zone_events_enqueue(zone_t *zone, zone_event_type_t type)
@@ -311,18 +332,7 @@ void zone_events_enqueue(zone_t *zone, zone_event_type_t type)
 	pthread_mutex_unlock(&events->mx);
 
 	/* Execute as soon as possible. */
-	zone_events_schedule(zone, type, ZONE_EVENT_NOW);
-}
-
-void zone_events_schedule(zone_t *zone, zone_event_type_t type, unsigned dt)
-{
-	time_t abstime = time(NULL) + dt;
-	zone_events_schedule_at(zone, type, abstime);
-}
-
-void zone_events_cancel(zone_t *zone, zone_event_type_t type)
-{
-	zone_events_schedule_at(zone, type, 0);
+	zone_events_schedule_now(zone, type);
 }
 
 void zone_events_freeze(zone_t *zone)
@@ -408,15 +418,3 @@ time_t zone_events_get_next(const struct zone *zone, zone_event_type_t *type)
 
 	return next_time;
 }
-
-void zone_events_update(conf_t *conf, zone_t *zone, zone_t *old_zone)
-{
-	replan_events(conf, zone, old_zone);
-}
-
-void zone_events_replan_ddns(zone_t *zone, zone_t *old_zone)
-{
-	if (old_zone) {
-		replan_update(zone, old_zone);
-	}
-}
