@@ -25,9 +25,6 @@
 #include "knot/worker/pool.h"
 #include "libknot/db/db.h"
 
-/* Timer special values. */
-#define ZONE_EVENT_NOW 0
-
 struct zone;
 
 typedef enum zone_event_type {
@@ -35,7 +32,6 @@ typedef enum zone_event_type {
 	// supported event types
 	ZONE_EVENT_LOAD = 0,
 	ZONE_EVENT_REFRESH,
-	ZONE_EVENT_XFER,
 	ZONE_EVENT_UPDATE,
 	ZONE_EVENT_EXPIRE,
 	ZONE_EVENT_FLUSH,
@@ -102,44 +98,31 @@ void zone_events_deinit(struct zone *zone);
 void zone_events_enqueue(struct zone *zone, zone_event_type_t type);
 
 /*!
- * \brief Schedule new zone event to absolute time.
+ * \brief Schedule new zone event.
+ *
+ * The function allows to set multiple events at once.
+ *
+ * The function intreprets time values (t) as follows:
+ *
+ *   t > 0: schedule timer for a given time
+ *   t = 0: cancel the timer
+ *   t < 0: ignore change in the timer
  *
  * If the event is already scheduled, the new time will be set only if the
- * new time is earlier than the currently scheduled one. An exception is
- * a zero time, which causes event cancellation.
+ * new time is earlier than the currently scheduled one. To override the
+ * check, cancel and schedule the event in a single function call.
  *
  * \param zone  Zone to schedule new event for.
- * \param type  Type of event.
- * \param time  Absolute time.
+ * \param ...   Sequence of zone_event_type_t and time_t terminated with
+ *              ZONE_EVENT_INVALID.
  */
-void zone_events_schedule_at(struct zone *zone, zone_event_type_t type, time_t time);
+void _zone_events_schedule_at(struct zone *zone, ...);
 
-/*!
- * \brief Schedule new zone event using relative time to current time.
- *
- * The function internally uses \ref zone_events_schedule_at.
- *
- * \param zone  Zone to schedule new event for.
- * \param type  Type of event.
- * \param dt    Relative time.
- */
-void zone_events_schedule(struct zone *zone, zone_event_type_t type, unsigned dt);
+#define zone_events_schedule_at(zone, events...) \
+	_zone_events_schedule_at(zone, events, ZONE_EVENT_INVALID)
 
-/*!
- * \brief Check if zone event is scheduled.
- *
- * \param zone  Zone to check event of.
- * \param type  Type of event.
- */
-bool zone_events_is_scheduled(struct zone *zone, zone_event_type_t type);
-
-/*!
- * \brief Cancel one zone event.
- *
- * \param zone  Zone to cancel event in.
- * \param type  Type of event to cancel.
- */
-void zone_events_cancel(struct zone *zone, zone_event_type_t type);
+#define zone_events_schedule_now(zone, type) \
+	zone_events_schedule_at(zone, type, time(NULL))
 
 /*!
  * \brief Freeze all zone events and prevent new events from running.
@@ -186,20 +169,3 @@ const char *zone_events_get_name(zone_event_type_t type);
  * \return time of the next event or an error (negative number)
  */
 time_t zone_events_get_next(const struct zone *zone, zone_event_type_t *type);
-
-/*!
- * \brief Replans zone events after config change. Will reuse events where applicable.
- *
- * \param conf      Configuration.
- * \param zone      Zone with new config.
- * \param old_zone  Zone with old config.
- */
-void zone_events_update(conf_t *conf, struct zone *zone, struct zone *old_zone);
-
-/*!
- * \brief Replans DDNS processing event if DDNS queue is not empty.
- *
- * \param zone      Zone with new config.
- * \param old_zone  Zone with old config.
- */
-void zone_events_replan_ddns(struct zone *zone, struct zone *old_zone);
