@@ -48,12 +48,8 @@ enum zonechecks_errors {
 	ZC_ERR_RRSIG_RDATA_EXPIRATION,
 	ZC_ERR_RRSIG_RDATA_LABELS,
 	ZC_ERR_RRSIG_RDATA_DNSKEY_OWNER,
-	ZC_ERR_RRSIG_NO_DNSKEY,
-	ZC_ERR_RRSIG_RDATA_SIGNED_WRONG,
 	ZC_ERR_RRSIG_NO_RRSIG,
 	ZC_ERR_RRSIG_SIGNED,
-	ZC_ERR_RRSIG_OWNER,
-	ZC_ERR_RRSIG_CLASS,
 	ZC_ERR_RRSIG_TTL,
 
 	ZC_ERR_RRSIG_GENERAL_ERROR, /* RRSIG error delimiter. */
@@ -62,13 +58,12 @@ enum zonechecks_errors {
 	ZC_ERR_NSEC_RDATA_BITMAP,
 	ZC_ERR_NSEC_RDATA_MULTIPLE,
 	ZC_ERR_NSEC_RDATA_CHAIN,
-	ZC_ERR_NSEC_RDATA_CHAIN_NOT_CYCLIC,
 
 	ZC_ERR_NSEC_GENERAL_ERROR, /* NSEC error delimiter. */
 
 	ZC_ERR_NSEC3_NOT_FOUND,
 	ZC_ERR_NSEC3_INSECURE_DELEGATION_OPT,
-	ZC_ERR_NSEC3_RDATA_TTL,
+	ZC_ERR_NSEC3_TTL,
 	ZC_ERR_NSEC3_RDATA_CHAIN,
 	ZC_ERR_NSEC3_EXTRA_RECORD,
 
@@ -76,7 +71,6 @@ enum zonechecks_errors {
 
 	ZC_ERR_CNAME_EXTRA_RECORDS,
 	ZC_ERR_DNAME_CHILDREN,
-	ZC_ERR_CNAME_EXTRA_RECORDS_DNSSEC,
 	ZC_ERR_CNAME_MULTIPLE,
 	ZC_ERR_DNAME_MULTIPLE,
 	ZC_ERR_CNAME_WILDCARD_SELF,
