@@ -33,15 +33,113 @@
 #include "contrib/ucw/lists.h"
 #include "contrib/ucw/mempool.h"
 
-#define JOURNAL_SUFFIX	".diff.db"
+#define JOURNAL_LOCK_MUTEX (&zone->journal_lock)
+#define JOURNAL_LOCK_RW pthread_mutex_lock(JOURNAL_LOCK_MUTEX);
+#define JOURNAL_UNLOCK_RW pthread_mutex_unlock(JOURNAL_LOCK_MUTEX);
 
-static void free_ddns_queue(zone_t *z)
+static void free_ddns_queue(zone_t *zone)
 {
 	ptrnode_t *node = NULL, *nxt = NULL;
-	WALK_LIST_DELSAFE(node, nxt, z->ddns_queue) {
+	WALK_LIST_DELSAFE(node, nxt, zone->ddns_queue) {
 		knot_request_free(node->d, NULL);
 	}
-	ptrlist_free(&z->ddns_queue, NULL);
+	ptrlist_free(&zone->ddns_queue, NULL);
+}
+
+/*! \brief Open journal for zone. */
+static int open_journal(zone_t *zone)
+{
+	assert(zone);
+
+	int ret = journal_open(zone->journal, zone->journal_db, zone->name);
+	if (ret != KNOT_EOK) {
+		log_zone_error(zone->name, "failed to open journal '%s'",
+		               (*zone->journal_db)->path);
+	}
+
+	return ret;
+}
+
+/*! \brief Close the zone journal. */
+static void close_journal(zone_t *zone)
+{
+	assert(zone);
+	journal_close(zone->journal);
+}
+
+static int flush_journal(conf_t *conf, zone_t *zone)
+{
+	/*! @note Function expects nobody will change zone contents meanwile. */
+
+	assert(zone);
+
+	bool force = zone->flags & ZONE_FORCE_FLUSH;
+	zone->flags &= ~ZONE_FORCE_FLUSH;
+
+	if (zone_contents_is_empty(zone->contents)) {
+		return KNOT_EINVAL;
+	}
+
+	/* Check for disabled zonefile synchronization. */
+	conf_val_t val = conf_zone_get(conf, C_ZONEFILE_SYNC, zone->name);
+	if (conf_int(&val) < 0 && !force) {
+		return KNOT_EOK;
+	}
+
+	/* Check for difference against zonefile serial. */
+	zone_contents_t *contents = zone->contents;
+	uint32_t serial_to = zone_contents_serial(contents);
+	if (!force && zone->zonefile.exists && zone->zonefile.serial == serial_to) {
+		return KNOT_EOK; /* No differences. */
+	}
+
+	char *zonefile = conf_zonefile(conf, zone->name);
+
+	/* Synchronize journal. */
+	int ret = zonefile_write(zonefile, contents);
+	if (ret != KNOT_EOK) {
+		log_zone_warning(zone->name, "failed to update zone file (%s)",
+		                 knot_strerror(ret));
+		free(zonefile);
+		return ret;
+	}
+
+	if (zone->zonefile.exists) {
+		log_zone_info(zone->name, "zone file updated, serial %u -> %u",
+		              zone->zonefile.serial, serial_to);
+	} else {
+		log_zone_info(zone->name, "zone file updated, serial %u",
+		              serial_to);
+	}
+
+	/* Update zone version. */
+	struct stat st;
+	if (stat(zonefile, &st) < 0) {
+		log_zone_warning(zone->name, "failed to update zone file (%s)",
+		                 knot_strerror(knot_map_errno()));
+		free(zonefile);
+		return KNOT_EACCES;
+	}
+
+	free(zonefile);
+
+	/* Update zone file serial and journal. */
+	zone->zonefile.exists = true;
+	zone->zonefile.mtime = st.st_mtime;
+	zone->zonefile.serial = serial_to;
+
+	/* Flush journal. */
+	if (zone->journal && journal_exists(zone->journal_db, zone->name)) {
+		ret = journal_flush(zone->journal);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+	}
+
+	/* Trim extra heap. */
+	mem_trim();
+
+	return ret;
 }
 
 zone_t* zone_new(const knot_dname_t *name)
@@ -58,6 +156,14 @@ zone_t* zone_new(const knot_dname_t *name)
 		return NULL;
 	}
 
+	// Journal
+	zone->journal = journal_new();
+	if (zone->journal == NULL) {
+		knot_dname_free(&zone->name, NULL);
+		free(zone);
+		return NULL;
+	}
+
 	// DDNS
 	pthread_mutex_init(&zone->ddns_lock, NULL);
 	zone->ddns_queue_size = 0;
@@ -97,10 +203,14 @@ void zone_free(zone_t **zone_ptr)
 
 	zone_t *zone = *zone_ptr;
 
+	close_journal(zone);
+
 	zone_events_deinit(zone);
 
 	knot_dname_free(&zone->name, NULL);
 
+	journal_free(&zone->journal);
+
 	free_ddns_queue(zone);
 	pthread_mutex_destroy(&zone->ddns_lock);
 	pthread_mutex_destroy(&zone->journal_lock);
@@ -127,24 +237,23 @@ int zone_change_store(conf_t *conf, zone_t *zone, changeset_t *change)
 		return KNOT_EINVAL;
 	}
 
-	conf_val_t val = conf_zone_get(conf, C_MAX_JOURNAL_SIZE, zone->name);
-	int64_t ixfr_fslimit = conf_int(&val);
-	char *journal_file = conf_journalfile(conf, zone->name);
+	JOURNAL_LOCK_RW
 
-	pthread_mutex_lock(&zone->journal_lock);
-	int ret = journal_store_changeset(change, journal_file, ixfr_fslimit);
-	if (ret == KNOT_EBUSY) {
-		log_zone_notice(zone->name, "journal is full, flushing");
+	int ret = open_journal(zone);
+	if (ret == KNOT_EOK) {
+		ret = journal_store_changeset(zone->journal, change);
+		if (ret == KNOT_EBUSY) {
+			log_zone_notice(zone->name, "journal is full, flushing");
 
-		/* Transaction rolled back, journal released, we may flush. */
-		ret = zone_flush_journal(conf, zone);
-		if (ret == KNOT_EOK) {
-			ret = journal_store_changeset(change, journal_file, ixfr_fslimit);
+			/* Transaction rolled back, journal released, we may flush. */
+			ret = flush_journal(conf, zone);
+			if (ret == KNOT_EOK) {
+				ret = journal_store_changeset(zone->journal, change);
+			}
 		}
 	}
-	pthread_mutex_unlock(&zone->journal_lock);
 
-	free(journal_file);
+	JOURNAL_UNLOCK_RW
 
 	return ret;
 }
@@ -155,25 +264,68 @@ int zone_changes_store(conf_t *conf, zone_t *zone, list_t *chgs)
 		return KNOT_EINVAL;
 	}
 
-	conf_val_t val = conf_zone_get(conf, C_MAX_JOURNAL_SIZE, zone->name);
-	int64_t ixfr_fslimit = conf_int(&val);
-	char *journal_file = conf_journalfile(conf, zone->name);
+	JOURNAL_LOCK_RW
 
-	pthread_mutex_lock(&zone->journal_lock);
-	int ret = journal_store_changesets(chgs, journal_file, ixfr_fslimit);
-	if (ret == KNOT_EBUSY) {
-		log_zone_notice(zone->name, "journal is full, flushing");
+	int ret = open_journal(zone);
+	if (ret == KNOT_EOK) {
+		ret = journal_store_changesets(zone->journal, chgs);
+		if (ret == KNOT_EBUSY) {
+			log_zone_notice(zone->name, "journal is full, flushing");
 
-		/* Transaction rolled back, journal released, we may flush. */
-		ret = zone_flush_journal(conf, zone);
-		if (ret == KNOT_EOK) {
-			ret = journal_store_changesets(chgs, journal_file, ixfr_fslimit);
+			/*
+			 * TODO: Check where zone_flush_journal is used beyond
+			 * event_flush(). zone->timers.last flush needs to be
+			 * updated and flush event rescheduled.
+			 *
+			 * The code bellow does only half of it.
+			 */
+
+			/* Transaction rolled back, journal released, we may flush. */
+			ret = flush_journal(conf, zone);
+			if (ret == KNOT_EOK) {
+				zone->timers.last_flush = time(NULL);
+				ret = journal_store_changesets(zone->journal, chgs);
+			}
 		}
-
 	}
-	pthread_mutex_unlock(&zone->journal_lock);
 
-	free(journal_file);
+	JOURNAL_UNLOCK_RW
+
+	return ret;
+}
+
+int zone_changes_load(conf_t *conf, zone_t *zone, list_t *dst, uint32_t from)
+{
+	if (conf == NULL || zone == NULL || dst == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	int ret = KNOT_ENOENT;
+
+	if (journal_exists(zone->journal_db, zone->name)) {
+		ret = open_journal(zone);
+	}
+
+	if (ret == KNOT_EOK) {
+		ret = journal_load_changesets(zone->journal, dst, from);
+	}
+
+	return ret;
+}
+
+int zone_flush_journal(conf_t *conf, zone_t *zone)
+{
+	if (conf == NULL || zone == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	JOURNAL_LOCK_RW
+
+	// NO open_journal() here.
+
+	int ret = flush_journal(conf, zone);
+
+	JOURNAL_UNLOCK_RW
 
 	return ret;
 }
@@ -226,6 +378,27 @@ void zone_clear_preferred_master(zone_t *zone)
 	pthread_mutex_unlock(&zone->preferred_lock);
 }
 
+const knot_rdataset_t *zone_soa(const zone_t *zone)
+{
+	if (!zone || zone_contents_is_empty(zone->contents)) {
+		return NULL;
+	}
+
+	return node_rdataset(zone->contents->apex, KNOT_RRTYPE_SOA);
+}
+
+bool zone_expired(const zone_t *zone)
+{
+	if (!zone) {
+		return false;
+	}
+
+	const zone_timers_t *timers = &zone->timers;
+
+	return timers->last_refresh > 0 && timers->soa_expire > 0 &&
+	       timers->last_refresh + timers->soa_expire <= time(NULL);
+}
+
 /*!
  * \brief Get preferred zone master while checking its existence.
  */
@@ -315,74 +488,6 @@ int zone_master_try(conf_t *conf, zone_t *zone, zone_master_cb callback,
 	return success ? KNOT_EOK : KNOT_ENOMASTER;
 }
 
-int zone_flush_journal(conf_t *conf, zone_t *zone)
-{
-	if (conf == NULL || zone == NULL || zone_contents_is_empty(zone->contents)) {
-		return KNOT_EINVAL;
-	}
-
-	bool force = zone->flags & ZONE_FORCE_FLUSH;
-	zone->flags &= ~ZONE_FORCE_FLUSH;
-
-	/* Check for disabled zonefile synchronization. */
-	conf_val_t val = conf_zone_get(conf, C_ZONEFILE_SYNC, zone->name);
-	if (conf_int(&val) < 0 && !force) {
-		return KNOT_EOK;
-	}
-
-	/* Check for difference against zonefile serial. */
-	zone_contents_t *contents = zone->contents;
-	uint32_t serial_to = zone_contents_serial(contents);
-	if (!force && zone->zonefile.exists && zone->zonefile.serial == serial_to) {
-		return KNOT_EOK; /* No differences. */
-	}
-
-	char *zonefile = conf_zonefile(conf, zone->name);
-
-	/* Synchronize journal. */
-	int ret = zonefile_write(zonefile, contents);
-	if (ret != KNOT_EOK) {
-		log_zone_warning(zone->name, "failed to update zone file (%s)",
-		                 knot_strerror(ret));
-		free(zonefile);
-		return ret;
-	}
-
-	if (zone->zonefile.exists) {
-		log_zone_info(zone->name, "zone file updated, serial %u -> %u",
-		              zone->zonefile.serial, serial_to);
-	} else {
-		log_zone_info(zone->name, "zone file updated, serial %u",
-		              serial_to);
-	}
-
-	/* Update zone version. */
-	struct stat st;
-	if (stat(zonefile, &st) < 0) {
-		log_zone_warning(zone->name, "failed to update zone file (%s)",
-		                 knot_strerror(knot_map_errno()));
-		free(zonefile);
-		return KNOT_EACCES;
-	}
-
-	free(zonefile);
-
-	char *journal_file = conf_journalfile(conf, zone->name);
-
-	/* Update zone file serial and journal. */
-	zone->zonefile.exists = true;
-	zone->zonefile.mtime = st.st_mtime;
-	zone->zonefile.serial = serial_to;
-	journal_mark_synced(journal_file);
-
-	free(journal_file);
-
-	/* Trim extra heap. */
-	mem_trim();
-
-	return ret;
-}
-
 int zone_update_enqueue(zone_t *zone, knot_pkt_t *pkt, struct process_query_param *param)
 {
 	if (zone == NULL || pkt == NULL || param == NULL) {
@@ -417,7 +522,7 @@ int zone_update_enqueue(zone_t *zone, knot_pkt_t *pkt, struct process_query_para
 	pthread_mutex_unlock(&zone->ddns_lock);
 
 	/* Schedule UPDATE event. */
-	zone_events_schedule(zone, ZONE_EVENT_UPDATE, ZONE_EVENT_NOW);
+	zone_events_schedule_now(zone, ZONE_EVENT_UPDATE);
 
 	return KNOT_EOK;
 }
@@ -444,27 +549,3 @@ size_t zone_update_dequeue(zone_t *zone, list_t *updates)
 
 	return update_count;
 }
-
-bool zone_transfer_needed(const zone_t *zone, const knot_pkt_t *pkt)
-{
-	if (zone == NULL || pkt == NULL) {
-		return false;
-	}
-
-	if (zone_contents_is_empty(zone->contents)) {
-		return true;
-	}
-
-	const knot_pktsection_t *answer = knot_pkt_section(pkt, KNOT_ANSWER);
-	if (answer->count < 1) {
-		return false;
-	}
-
-	const knot_rrset_t *soa = knot_pkt_rr(answer, 0);
-	if (soa->type != KNOT_RRTYPE_SOA) {
-		return false;
-	}
-
-	return serial_compare(zone_contents_serial(zone->contents),
-	                           knot_soa_serial(&soa->rrs)) < 0;
-}
