@@ -25,8 +25,7 @@
  * @{
  */
 
-#ifndef _KNOT_BINARY_H_
-#define _KNOT_BINARY_H_
+#pragma once
 
 #include <stdlib.h>
 #include <stddef.h>
@@ -77,6 +76,4 @@ int knot_binary_free(knot_binary_t *binary);
  */
 int knot_binary_dup(const knot_binary_t *from, knot_binary_t *to);
 
-#endif // _KNOT_BINARY_H
-
 /*! @} */
