@@ -24,19 +24,18 @@
 
 #pragma once
 
-#include "contrib/hat-trie/hat-trie.h"
+#include "contrib/qp-trie/trie.h"
 #include "zscanner/scanner.h"
 
 /*!
  * \brief Memory estimation context.
  */
 typedef struct {
-	hattrie_t *node_table; /*!< Same trie is in actual zone. */
-	size_t rdata_size;     /*!< Estimated RDATA size. */
-	size_t dname_size;     /*!< Estimated DNAME size. */
-	size_t node_size;      /*!< Estimated node size. */
-	size_t htable_size;    /*!< Estimated ahtable size. */
-	size_t record_count;   /*!< Total record count for zone. */
+	trie_t *node_table;  /*!< Same trie is in actual zone. */
+	size_t rdata_size;   /*!< Estimated RDATA size. */
+	size_t dname_size;   /*!< Estimated DNAME size. */
+	size_t node_size;    /*!< Estimated node size. */
+	size_t record_count; /*!< Total record count for zone. */
 } zone_estim_t;
 
 /*!
@@ -57,13 +56,6 @@ void *estimator_malloc(void *ctx, size_t len);
  */
 void estimator_free(void *p);
 
-/*!
- * \brief Goes through trie's ahtables and estimates their memory requirements.
- *
- * \param table Trie to traverse.
- */
-size_t estimator_trie_htable_memsize(hattrie_t *table);
-
 /*!
  * \brief For use with scanner - counts memsize of RRSets.
  *
@@ -72,10 +64,10 @@ size_t estimator_trie_htable_memsize(hattrie_t *table);
 void estimator_rrset_memsize_wrap(zs_scanner_t *scanner);
 
 /*!
- * \brief Cleanup function for use with hattrie.
+ * \brief Cleanup function for use with trie.
  *
  * \param p Data to free.
  */
-int estimator_free_trie_node(value_t *val, void *data);
+int estimator_free_trie_node(trie_val_t *val, void *data);
 
 /*! @} */
